@@ -4,7 +4,7 @@ use std::env;
 /**
  * Simple utility to print distance between two songs according to bliss.
  *
- * Takes two file paths, and analyse the corresponding songs, printing
+ * Takes two file paths, and analyze the corresponding songs, printing
  * the distance between the two files according to bliss.
  */
 fn main() -> Result<(), String> {
@@ -13,8 +13,8 @@ fn main() -> Result<(), String> {
     let first_path = paths.next().ok_or("Help: ./distance <song1> <song2>")?;
     let second_path = paths.next().ok_or("Help: ./distance <song1> <song2>")?;
 
-    let song1 = Song::new(&first_path).map_err(|x| x.to_string())?;
-    let song2 = Song::new(&second_path).map_err(|x| x.to_string())?;
+    let song1 = Song::from_path(&first_path).map_err(|x| x.to_string())?;
+    let song2 = Song::from_path(&second_path).map_err(|x| x.to_string())?;
 
     println!(
         "d({:?}, {:?}) = {}",
