@@ -207,13 +207,13 @@ fn chroma_filter(
     wts *= &freq_bins;
 
     // np.roll(), np bro
-    let mut uninit: Vec<f64> = Vec::with_capacity((&wts).len());
+    let mut uninit: Vec<f64> = vec![0.; (&wts).len()];
     unsafe {
         uninit.set_len(wts.len());
     }
     let mut b = Array::from(uninit)
         .into_shape(wts.dim())
-        .map_err(|e| BlissError::AnalysisError(format!("in chroma: {}", e.to_string())))?;
+        .map_err(|e| BlissError::AnalysisError(format!("in chroma: {}", e)))?;
     b.slice_mut(s![-3.., ..]).assign(&wts.slice(s![..3, ..]));
     b.slice_mut(s![..-3, ..]).assign(&wts.slice(s![3.., ..]));
 
@@ -308,7 +308,7 @@ fn pitch_tuning(
     }
     let max_index = counts
         .argmax()
-        .map_err(|e| BlissError::AnalysisError(format!("in chroma: {}", e.to_string())))?;
+        .map_err(|e| BlissError::AnalysisError(format!("in chroma: {}", e)))?;
 
     // Return the bin with the most reoccuring frequency.
     Ok((-50. + (100. * resolution * max_index as f64)) / 100.)
@@ -330,11 +330,14 @@ fn estimate_tuning(
         .map(|(x, y)| (n64(*x), n64(*y)))
         .unzip();
 
+    if pitch.is_empty() {
+        return Ok(0.);
+    }
+
     let threshold: N64 = Array::from(filtered_mag.to_vec())
         .quantile_axis_mut(Axis(0), n64(0.5), &Midpoint)
-        .map_err(|e| BlissError::AnalysisError(format!("in chroma: {}", e.to_string())))?
+        .map_err(|e| BlissError::AnalysisError(format!("in chroma: {}", e)))?
         .into_scalar();
-
     let mut pitch = filtered_pitch
         .iter()
         .zip(&filtered_mag)
@@ -486,6 +489,11 @@ mod test {
         assert!(0.000001 > (-0.09999999999999998 - tuning).abs());
     }
 
+    #[test]
+    fn test_chroma_estimate_tuning_empty_fix() {
+        assert!(0. == estimate_tuning(22050, &Array2::zeros((8192, 1)), 8192, 0.01, 12).unwrap());
+    }
+
     #[test]
     fn test_estimate_tuning_decode() {
         let signal = Song::decode(Path::new("data/s16_mono_22_5kHz.flac"))
