@@ -12,13 +12,16 @@ extern crate ffmpeg_next as ffmpeg;
 extern crate ndarray;
 extern crate ndarray_npy;
 
-use crate::{CHANNELS, FEATURES_VERSION};
 use crate::chroma::ChromaDesc;
-use crate::distance::{euclidean_distance, DistanceMetric};
+use crate::cue::CueInfo;
 use crate::misc::LoudnessDesc;
+#[cfg(doc)]
+use crate::playlist;
+use crate::playlist::{closest_to_first_song, dedup_playlist, euclidean_distance, DistanceMetric};
 use crate::temporal::BPMDesc;
 use crate::timbral::{SpectralDesc, ZeroCrossingRateDesc};
 use crate::{BlissError, BlissResult, SAMPLE_RATE};
+use crate::{CHANNELS, FEATURES_VERSION};
 use ::log::warn;
 use core::ops::Index;
 use crossbeam::thread;
@@ -39,6 +42,7 @@ use std::path::PathBuf;
 use std::sync::mpsc;
 use std::sync::mpsc::Receiver;
 use std::thread as std_thread;
+use std::time::Duration;
 use strum::{EnumCount, IntoEnumIterator};
 use strum_macros::{EnumCount, EnumIter};
 
@@ -55,16 +59,26 @@ pub struct Song {
     pub title: Option<String>,
     /// Song's album name, read from the metadata
     pub album: Option<String>,
+    /// Song's album's artist name, read from the metadata
+    pub album_artist: Option<String>,
     /// Song's tracked number, read from the metadata
     pub track_number: Option<String>,
     /// Song's genre, read from the metadata (`""` if empty)
     pub genre: Option<String>,
     /// bliss analysis results
     pub analysis: Analysis,
+    /// The song's duration
+    pub duration: Duration,
     /// Version of the features the song was analyzed with.
     /// A simple integer that is bumped every time a breaking change
     /// is introduced in the features.
     pub features_version: u16,
+    /// Populated only if the song was extracted from a larger audio file,
+    /// through the use of a CUE sheet.
+    /// By default, such a song's path would be
+    /// `path/to/cue_file.wav/CUE_TRACK00<track_number>`. Using this field,
+    /// you can change `song.path` to fit your needs.
+    pub cue_info: Option<CueInfo>,
 }
 
 #[derive(Debug, EnumIter, EnumCount)]
@@ -75,7 +89,7 @@ pub struct Song {
 /// use bliss_audio::{AnalysisIndex, BlissResult, Song};
 ///
 /// fn main() -> BlissResult<()> {
-///     let song = Song::new("path/to/song")?;
+///     let song = Song::from_path("path/to/song")?;
 ///     println!("{}", song.analysis[AnalysisIndex::Tempo]);
 ///     Ok(())
 /// }
@@ -128,7 +142,7 @@ pub const NUMBER_FEATURES: usize = AnalysisIndex::COUNT;
 /// on most of the features, except the chroma ones, which are documented
 /// directly in this code.
 pub struct Analysis {
-    internal_analysis: [f32; NUMBER_FEATURES],
+    pub(crate) internal_analysis: [f32; NUMBER_FEATURES],
 }
 
 impl Index<AnalysisIndex> for Analysis {
@@ -228,12 +242,50 @@ impl Song {
         self.analysis.custom_distance(&other.analysis, distance)
     }
 
-    /// Returns a decoded Song given a file path, or an error if the song
+    /// Orders songs in `pool` by proximity to `self`, using the distance
+    /// metric `distance` to compute the order.
+    /// Basically return a playlist from songs in `pool`, starting
+    /// from `self`, using `distance` (some distance metrics can
+    /// be found in the [playlist] module).
+    ///
+    /// Note that contrary to [Song::closest_from_pool], `self` is NOT added
+    /// to the beginning of the returned vector.
+    ///
+    /// No deduplication is ran either; if you're looking for something easy
+    /// that works "out of the box", use [Song::closest_from_pool].
+    pub fn closest_from_pool_custom(
+        &self,
+        pool: Vec<Self>,
+        distance: impl DistanceMetric,
+    ) -> Vec<Self> {
+        let mut pool = pool;
+        closest_to_first_song(self, &mut pool, distance);
+        pool
+    }
+
+    /// Order songs in `pool` by proximity to `self`.
+    /// Convenience method to return a playlist from songs in `pool`,
+    /// starting from `self`.
+    ///
+    /// The distance is already chosen, deduplication is ran, and the first song
+    /// is added to the top of the playlist, to make everything easier.
+    ///
+    /// If you want more control over which distance metric is chosen,
+    /// run deduplication manually, etc, use [Song::closest_from_pool_custom].
+    pub fn closest_from_pool(&self, pool: Vec<Self>) -> Vec<Self> {
+        let mut playlist = vec![self.to_owned()];
+        playlist.extend_from_slice(&pool);
+        closest_to_first_song(self, &mut playlist, euclidean_distance);
+        dedup_playlist(&mut playlist, None);
+        playlist
+    }
+
+    /// Returns a decoded [Song] given a file path, or an error if the song
     /// could not be analyzed for some reason.
     ///
     /// # Arguments
     ///
-    /// * `path` - A string holding a valid file path to a valid audio file.
+    /// * `path` - A [Path] holding a valid file path to a valid audio file.
     ///
     /// # Errors
     ///
@@ -244,23 +296,26 @@ impl Song {
     /// The error type returned should give a hint as to whether it was a
     /// decoding ([DecodingError](BlissError::DecodingError)) or an analysis
     /// ([AnalysisError](BlissError::AnalysisError)) error.
-    pub fn new<P: AsRef<Path>>(path: P) -> BlissResult<Self> {
+    pub fn from_path<P: AsRef<Path>>(path: P) -> BlissResult<Self> {
         let raw_song = Song::decode(path.as_ref())?;
 
         Ok(Song {
             path: raw_song.path,
             artist: raw_song.artist,
+            album_artist: raw_song.album_artist,
             title: raw_song.title,
             album: raw_song.album,
             track_number: raw_song.track_number,
             genre: raw_song.genre,
-            analysis: Song::analyse(raw_song.sample_array)?,
+            duration: raw_song.duration,
+            analysis: Song::analyze(&raw_song.sample_array)?,
             features_version: FEATURES_VERSION,
+            cue_info: None,
         })
     }
 
     /**
-     * Analyse a song decoded in `sample_array`, with one channel @ 22050 Hz.
+     * Analyze a song decoded in `sample_array`, with one channel @ 22050 Hz.
      *
      * The current implementation doesn't make use of it,
      * but the song can also be streamed wrt.
@@ -270,7 +325,7 @@ impl Song {
      * Useful in the rare cases where the full song is not
      * completely available.
      **/
-    fn analyse(sample_array: Vec<f32>) -> BlissResult<Analysis> {
+    pub(crate) fn analyze(sample_array: &[f32]) -> BlissResult<Analysis> {
         let largest_window = vec![
             BPMDesc::WINDOW_SIZE,
             ChromaDesc::WINDOW_SIZE,
@@ -301,7 +356,7 @@ impl Song {
 
             let child_chroma: thread::ScopedJoinHandle<'_, BlissResult<Vec<f32>>> = s.spawn(|_| {
                 let mut chroma_desc = ChromaDesc::new(SAMPLE_RATE, 12);
-                chroma_desc.do_(&sample_array)?;
+                chroma_desc.do_(sample_array)?;
                 Ok(chroma_desc.get_values())
             });
 
@@ -325,7 +380,7 @@ impl Song {
 
             let child_zcr: thread::ScopedJoinHandle<'_, BlissResult<f32>> = s.spawn(|_| {
                 let mut zcr_desc = ZeroCrossingRateDesc::default();
-                zcr_desc.do_(&sample_array);
+                zcr_desc.do_(sample_array);
                 Ok(zcr_desc.get_value())
             });
 
@@ -366,29 +421,55 @@ impl Song {
     }
 
     pub(crate) fn decode(path: &Path) -> BlissResult<InternalSong> {
-        ffmpeg::init()
-            .map_err(|e| BlissError::DecodingError(format!("ffmpeg init error: {:?}.", e)))?;
+        ffmpeg::init().map_err(|e| {
+            BlissError::DecodingError(format!(
+                "ffmpeg init error while decoding file '{}': {:?}.",
+                path.display(),
+                e
+            ))
+        })?;
         log::set_level(Level::Quiet);
         let mut song = InternalSong {
             path: path.into(),
             ..Default::default()
         };
-        let mut format = ffmpeg::format::input(&path)
-            .map_err(|e| BlissError::DecodingError(format!("while opening format: {:?}.", e)))?;
+        let mut format = ffmpeg::format::input(&path).map_err(|e| {
+            BlissError::DecodingError(format!(
+                "while opening format for file '{}': {:?}.",
+                path.display(),
+                e
+            ))
+        })?;
         let (mut codec, stream, expected_sample_number) = {
             let stream = format
                 .streams()
-                .find(|s| s.codec().medium() == ffmpeg::media::Type::Audio)
-                .ok_or_else(|| BlissError::DecodingError(String::from("No audio stream found.")))?;
-            stream.codec().set_threading(Config {
+                .find(|s| s.parameters().medium() == ffmpeg::media::Type::Audio)
+                .ok_or_else(|| {
+                    BlissError::DecodingError(format!(
+                        "No audio stream found for file '{}'.",
+                        path.display()
+                    ))
+                })?;
+            let mut context = ffmpeg::codec::context::Context::from_parameters(stream.parameters())
+                .map_err(|e| {
+                    BlissError::DecodingError(format!(
+                        "Could not load the codec context for file '{}': {:?}",
+                        path.display(),
+                        e
+                    ))
+                })?;
+            context.set_threading(Config {
                 kind: ThreadingType::Frame,
                 count: 0,
                 safe: true,
             });
-            let codec =
-                stream.codec().decoder().audio().map_err(|e| {
-                    BlissError::DecodingError(format!("when finding codec: {:?}.", e))
-                })?;
+            let codec = context.decoder().audio().map_err(|e| {
+                BlissError::DecodingError(format!(
+                    "when finding codec for file '{}': {:?}.",
+                    path.display(),
+                    e
+                ))
+            })?;
             // Add SAMPLE_RATE to have one second margin to avoid reallocating if
             // the duration is slightly more than estimated
             // TODO>1.0 another way to get the exact number of samples is to decode
@@ -433,6 +514,13 @@ impl Song {
                 t => Some(t.to_string()),
             };
         };
+        if let Some(album_artist) = format.metadata().get("album_artist") {
+            song.album_artist = match album_artist {
+                "" => None,
+                t => Some(t.to_string()),
+            };
+        };
+
         let in_channel_layout = {
             if codec.channel_layout() == ChannelLayout::empty() {
                 ChannelLayout::default(codec.channels().into())
@@ -461,17 +549,21 @@ impl Song {
             match codec.send_packet(&packet) {
                 Ok(_) => (),
                 Err(Error::Other { errno: EINVAL }) => {
-                    return Err(BlissError::DecodingError(String::from(
-                        "wrong codec opened.",
+                    return Err(BlissError::DecodingError(format!(
+                        "wrong codec opened for file '{}.",
+                        path.display(),
                     )))
                 }
                 Err(Error::Eof) => {
-                    warn!("Premature EOF reached while decoding.");
+                    warn!(
+                        "Premature EOF reached while decoding file '{}'.",
+                        path.display()
+                    );
                     drop(tx);
                     song.sample_array = child.join().unwrap()?;
                     return Ok(song);
                 }
-                Err(e) => warn!("error while decoding {}: {}", path.display(), e),
+                Err(e) => warn!("error while decoding file '{}': {}", path.display(), e),
             };
 
             loop {
@@ -480,8 +572,9 @@ impl Song {
                     Ok(_) => {
                         tx.send(decoded).map_err(|e| {
                             BlissError::DecodingError(format!(
-                                "while sending decoded frame to the resampling thread: {:?}",
-                                e
+                                "while sending decoded frame to the resampling thread for file '{}': {:?}",
+                                path.display(),
+                                e,
                             ))
                         })?;
                     }
@@ -495,12 +588,16 @@ impl Song {
         match codec.send_packet(&packet) {
             Ok(_) => (),
             Err(Error::Other { errno: EINVAL }) => {
-                return Err(BlissError::DecodingError(String::from(
-                    "wrong codec opened.",
+                return Err(BlissError::DecodingError(format!(
+                    "wrong codec opened for file '{}'.",
+                    path.display()
                 )))
             }
             Err(Error::Eof) => {
-                warn!("Premature EOF reached while decoding.");
+                warn!(
+                    "Premature EOF reached while decoding file '{}'.",
+                    path.display()
+                );
                 drop(tx);
                 song.sample_array = child.join().unwrap()?;
                 return Ok(song);
@@ -514,7 +611,8 @@ impl Song {
                 Ok(_) => {
                     tx.send(decoded).map_err(|e| {
                         BlissError::DecodingError(format!(
-                            "while sending decoded frame to the resampling thread: {:?}",
+                            "while sending decoded frame to the resampling thread for file '{}': {:?}",
+                            path.display(),
                             e
                         ))
                     })?;
@@ -525,6 +623,8 @@ impl Song {
 
         drop(tx);
         song.sample_array = child.join().unwrap()?;
+        let duration_seconds = song.sample_array.len() as f32 / SAMPLE_RATE as f32;
+        song.duration = Duration::from_nanos((duration_seconds * 1e9_f32).round() as u64);
         Ok(song)
     }
 }
@@ -533,10 +633,12 @@ impl Song {
 pub(crate) struct InternalSong {
     pub path: PathBuf,
     pub artist: Option<String>,
+    pub album_artist: Option<String>,
     pub title: Option<String>,
     pub album: Option<String>,
     pub track_number: Option<String>,
     pub genre: Option<String>,
+    pub duration: Duration,
     pub sample_array: Vec<f32>,
 }
 
@@ -616,18 +718,19 @@ fn push_to_sample_array(frame: &ffmpeg::frame::Audio, sample_array: &mut Vec<f32
 #[cfg(test)]
 mod tests {
     use super::*;
+    use pretty_assertions::assert_eq;
     use ripemd160::{Digest, Ripemd160};
     use std::path::Path;
 
     #[test]
     fn test_analysis_too_small() {
-        let error = Song::analyse(vec![0.]).unwrap_err();
+        let error = Song::analyze(&[0.]).unwrap_err();
         assert_eq!(
             error,
             BlissError::AnalysisError(String::from("empty or too short song."))
         );
 
-        let error = Song::analyse(vec![]).unwrap_err();
+        let error = Song::analyze(&[]).unwrap_err();
         assert_eq!(
             error,
             BlissError::AnalysisError(String::from("empty or too short song."))
@@ -635,8 +738,8 @@ mod tests {
     }
 
     #[test]
-    fn test_analyse() {
-        let song = Song::new(Path::new("data/s16_mono_22_5kHz.flac")).unwrap();
+    fn test_analyze() {
+        let song = Song::from_path(Path::new("data/s16_mono_22_5kHz.flac")).unwrap();
         let expected_analysis = vec![
             0.3846389,
             -0.849141,
@@ -679,10 +782,17 @@ mod tests {
     fn test_tags() {
         let song = Song::decode(Path::new("data/s16_mono_22_5kHz.flac")).unwrap();
         assert_eq!(song.artist, Some(String::from("David TMX")));
+        assert_eq!(
+            song.album_artist,
+            Some(String::from("David TMX - Album Artist"))
+        );
         assert_eq!(song.title, Some(String::from("Renaissance")));
         assert_eq!(song.album, Some(String::from("Renaissance")));
         assert_eq!(song.track_number, Some(String::from("02")));
         assert_eq!(song.genre, Some(String::from("Pop")));
+        // Test that there is less than 10ms of difference between what
+        // the song advertises and what we compute.
+        assert!((song.duration.as_millis() as f32 - 11070.).abs() < 10.);
     }
 
     #[test]
@@ -805,25 +915,27 @@ mod tests {
         assert_eq!(
             Song::decode(Path::new("nonexistent")).unwrap_err(),
             BlissError::DecodingError(String::from(
-                "while opening format: ffmpeg::Error(2: No such file or directory)."
+                "while opening format for file 'nonexistent': ffmpeg::Error(2: No such file or directory)."
             )),
         );
         assert_eq!(
             Song::decode(Path::new("data/picture.png")).unwrap_err(),
-            BlissError::DecodingError(String::from("No audio stream found.")),
+            BlissError::DecodingError(String::from(
+                "No audio stream found for file 'data/picture.png'."
+            )),
         );
     }
 
     #[test]
     fn test_index_analysis() {
-        let song = Song::new("data/s16_mono_22_5kHz.flac").unwrap();
+        let song = Song::from_path("data/s16_mono_22_5kHz.flac").unwrap();
         assert_eq!(song.analysis[AnalysisIndex::Tempo], 0.3846389);
         assert_eq!(song.analysis[AnalysisIndex::Chroma10], -0.95968974);
     }
 
     #[test]
     fn test_debug_analysis() {
-        let song = Song::new("data/s16_mono_22_5kHz.flac").unwrap();
+        let song = Song::from_path("data/s16_mono_22_5kHz.flac").unwrap();
         assert_eq!(
             "Analysis { Tempo: 0.3846389, Zcr: -0.849141, MeanSpectralCentroid: \
             -0.75481045, StdDeviationSpectralCentroid: -0.8790748, MeanSpectralR\
@@ -844,6 +956,7 @@ mod tests {
     fn dummy_distance(_: &Array1<f32>, _: &Array1<f32>) -> f32 {
         0.
     }
+
     #[test]
     fn test_custom_distance() {
         let mut a = Song::default();
@@ -861,6 +974,84 @@ mod tests {
         ]);
         assert_eq!(a.custom_distance(&b, dummy_distance), 0.);
     }
+
+    #[test]
+    fn test_closest_from_pool() {
+        let song = Song {
+            path: Path::new("path-to-first").to_path_buf(),
+            analysis: Analysis::new([
+                1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
+            ]),
+            ..Default::default()
+        };
+        let first_song_dupe = Song {
+            path: Path::new("path-to-dupe").to_path_buf(),
+            analysis: Analysis::new([
+                1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1., 1.,
+            ]),
+            ..Default::default()
+        };
+
+        let second_song = Song {
+            path: Path::new("path-to-second").to_path_buf(),
+            analysis: Analysis::new([
+                2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 1.9, 1., 1., 1.,
+            ]),
+            ..Default::default()
+        };
+        let third_song = Song {
+            path: Path::new("path-to-third").to_path_buf(),
+            analysis: Analysis::new([
+                2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2.5, 1., 1., 1.,
+            ]),
+            ..Default::default()
+        };
+        let fourth_song = Song {
+            path: Path::new("path-to-fourth").to_path_buf(),
+            analysis: Analysis::new([
+                2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 0., 1., 1., 1.,
+            ]),
+            ..Default::default()
+        };
+        let fifth_song = Song {
+            path: Path::new("path-to-fifth").to_path_buf(),
+            analysis: Analysis::new([
+                2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 2., 0., 1., 1., 1.,
+            ]),
+            ..Default::default()
+        };
+
+        let songs = vec![
+            song.to_owned(),
+            first_song_dupe.to_owned(),
+            second_song.to_owned(),
+            third_song.to_owned(),
+            fourth_song.to_owned(),
+            fifth_song.to_owned(),
+        ];
+        let playlist = song.closest_from_pool(songs.to_owned());
+        assert_eq!(
+            playlist,
+            vec![
+                song.to_owned(),
+                second_song.to_owned(),
+                fourth_song.to_owned(),
+                third_song.to_owned(),
+            ],
+        );
+        let playlist = song.closest_from_pool_custom(songs, euclidean_distance);
+        assert_eq!(
+            playlist,
+            vec![
+                song,
+                first_song_dupe,
+                second_song,
+                fourth_song,
+                fifth_song,
+                third_song
+            ],
+        );
+    }
 }
 
 #[cfg(all(feature = "bench", test))]
